#include "general.h"
#include "bitbang.h"
#include "t3_util.h"

#define SERIAL_WRITE (u08)0xa0
#define SERIAL_READ  (u08)0xa1
#define SERIAL_WAIT_H 3
#define SERIAL_WAIT_L 2

static void (*setClockFunc)(int);
static void (*setDataFunc)(int);
static int (*getDataFunc)(void);
static void (*setCSFunc)(int);

/****************************************************************************************/
/****************************************************************************************/
/*   ATMEL EEPROM bit-bang code                                                         */
/****************************************************************************************/
/****************************************************************************************/

static void eepStart(void)
{
    (setClockFunc)(1);
    util_wait_1us(SERIAL_WAIT_H);
    (setDataFunc)(0);
    util_wait_1us(SERIAL_WAIT_H);
    (setClockFunc)(0);
    util_wait_1us(SERIAL_WAIT_L);
}

/******************************************************/
static void eepStop()
{       
    (setDataFunc)(0);
    (setClockFunc)(1);
    util_wait_1us(SERIAL_WAIT_L);
    (setDataFunc)(1);
    util_wait_1us(SERIAL_WAIT_L);
}

/******************************************************/
static int eepReadBit(void)
{
    int data;
    
    util_wait_1us(SERIAL_WAIT_L);
    (setClockFunc)(1);
    util_wait_1us(SERIAL_WAIT_H);
    data = (getDataFunc)() ? 1 : 0;
    util_wait_1us(SERIAL_WAIT_H);
    (setClockFunc)(0);
    util_wait_1us(SERIAL_WAIT_L);
    return data;    
}

/******************************************************/
static void eepWriteBit(int data)
{
    (setDataFunc)(data?1:0);
    util_wait_1us(SERIAL_WAIT_L);
    (setClockFunc)(1);
    util_wait_1us(SERIAL_WAIT_H*2);
    (setClockFunc)(0);
    util_wait_1us(SERIAL_WAIT_L);
}

/******************************************************/
static u8 eepRead(int last)
{
    int i;
    u8 data;
    
    data = 0;
    for (i=0;i<8;i++)
        data = (data << 1) | eepReadBit();

    eepWriteBit(last);   /* we ack this byte */ 
    (setDataFunc)(1);  /* release data line */
    return data;   
}

/******************************************************/
static int eepWrite(u8 data)
{
    int i;
    
    for (i=0;i<8;i++)
    {
        eepWriteBit(data & 0x80);
        data <<= 1;
    }
    return eepReadBit() == 0;
}

/******************************************************/
static int eepCmd(u08 dev, u08 cmd)
{
    dev = (dev & 0x3) << 1;                          /* shift to dev position */
    eepStart();                                     /* generate start condition */
    return eepWrite(dev | cmd);                     /* write command */
}

/******************************************************/
static int eepSetupAddr(u08 dev, u16 addr)
{
    if (eepCmd(dev, SERIAL_WRITE))
        if (eepWrite((addr >> 8) & 0xff))
            if (eepWrite(addr & 0xff))
                return 1;
    return 0;
}    


/******************************************************/
/*    ATMEL EEPROM Public Routine                     */
/******************************************************/
int eepBitBangReadWord(u8 dev, u16 addr, u32* data, 
    void (*setClock)(int), 
    void (*setData)(int), 
    int (*getData)(void))
{
    u32 i;
    int retc = HAS_ERROR;

    setClockFunc = setClock;
    setDataFunc = setData;
    getDataFunc = getData;    
    if (eepSetupAddr(dev, addr))   /* do a dummy write to setup the address */
    {
        if (eepCmd(dev, SERIAL_READ))
        {
            *data = 0;
            for (i=0;i<4;i++)
                *data = (*data << 8) + (u32) eepRead(i==3);
            *data = BYTE_SWAP(*data);    
            retc = NO_ERROR;    
        }    
    }
    eepStop();
    return retc;
}    

/******************************************************/
int eepBitBangWriteWord(u8 dev, u16 addr, u32 data, void (*setClock)(int), void (*setData)(int), int (*getData)(void))
{
    u32 i;
    int retc = HAS_ERROR;

    setClockFunc = setClock;
    setDataFunc = setData;
    getDataFunc = getData; 
    data = BYTE_SWAP(data);   
    if (eepSetupAddr(dev, addr))   /* do a dummy write to setup the address */
    {
        for (i=0;i<4;i++)
        {
            if (!eepWrite(data & 0xff))
            {
                retc = HAS_ERROR;
                break;
            }    
            data >>= 8;
            retc = NO_ERROR;
        }    
    }
    eepStop();
    return retc;
}    


/****************************************************************************************/
/****************************************************************************************/
/*   Common code for FLASH bit bang                                                     */
/****************************************************************************************/
/****************************************************************************************/
static void flshWriteByte(u08 byte)
{
    int i;
    
    for (i=0;i<8;i++)
    {
        (setDataFunc)(byte & 0x80 ? 1 : 0);
        (setClockFunc)(1); 
        (setClockFunc)(0);
        byte <<= 1;
    }
}

/******************************************************/
static u32 flshBangReadbit(void)
{
    u32 data;
    
    (setClockFunc)(1); 
    data = (getDataFunc)() ? 1 : 0;
    (setClockFunc)(0);
    return data; 
}

/******************************************************/
static u32 flshBangRead1(void)
{
    u32 data = 0;
    int i;
    
    for (i=0;i<8;i++)
    {
        data <<= 1;
        data |= flshBangReadbit();
    }
    return data;
}

/******************************************************/
static u32 flshBangRead4(void)
{
    u32 data = 0;
    int i;
    
    for (i=0;i<4;i++)
    {
        data <<= 8;
        data |= flshBangRead1();
    }
    return data;
}

/****************************************************************************************/
/****************************************************************************************/
/*   SST FLASH bit bang code                                                            */
/****************************************************************************************/
/****************************************************************************************/
#define FLASH_SST_CMD_READ          0xff
#define FLASH_SST_CMD_SECT_ERASE    0x20
#define FLASH_SST_CMD_CHIP_ERASE    0x60
#define FLASH_SST_CMD_BYTE_PROGRAM  0x10
#define FLASH_SST_CMD_SOFT_STATUS   0x9f
#define FLASH_SST_CMD_READ_ID       0x90

#define FLASH_SST_ERASE_DATA        0xd0

/******************************************************/
/*    Private Routine                                 */
/******************************************************/
/******************************************************/
static void sstBangCmd(u08 cmd, u32 addr, u08 data)
{
    flshWriteByte(cmd);
    flshWriteByte((u08) addr>>16);
    flshWriteByte((u08) addr>>8);
    flshWriteByte((u08) addr);
    flshWriteByte((u08) addr);
    flshWriteByte(data);
    flshWriteByte(0);       /* dummy */
}    

/******************************************************/
static int sstBangStatus(void)
{
    int status=HAS_ERROR;
    int wait = 0x1000000;
    
    (setCSFunc)(1);                /* enable chip select */
    flshWriteByte(FLASH_SST_CMD_SOFT_STATUS);
    while (wait--)
    {
        if (flshBangRead1() & 1)
        {
            status = NO_ERROR;
            break;
        }
    }
    (setCSFunc)(0);
    return status;
}

/******************************************************/
static int sstCmdWriteByte(u32 addr, u08 data)
{
    (setCSFunc)(1);                /* enable chip select */
    sstBangCmd(FLASH_SST_CMD_BYTE_PROGRAM,addr,data);
    (setCSFunc)(0);
    return sstBangStatus();
}

/******************************************************/
/*    SST Public Routine                              */
/******************************************************/
int sstBitBangSectErase(u32 addr,
        void (*setClock)(int), 
        void (*setData)(int), 
        int (*getData)(void),  
        int (*setCS)(void)) 
{
    setClockFunc = setClock;
    setDataFunc = setData;
    getDataFunc = getData;    
    setCSFunc = setCS;
    (setCSFunc)(1);                /* enable chip select */
    sstBangCmd(FLASH_SST_CMD_SECT_ERASE,addr,FLASH_SST_ERASE_DATA);
    (setCSFunc)(0);
    return sstBangStatus();
}

/******************************************************/
int sstBitBangReadWord(u32 addr, u32* data, u32 WordCnt,
    void (*setClock)(int), 
    void (*setData)(int), 
    int (*getData)(void),  
    int (*setCS)(void))
{
    setClockFunc = setClock;
    setDataFunc = setData;
    getDataFunc = getData;    
    setCSFunc = setCS;
    (setCSFunc)(1);                /* enable chip select */
    sstBangCmd(FLASH_SST_CMD_READ,addr,0x00);
    while (WordCnt--)
        *data++ = flshBangRead4();
    (setCSFunc)(0);
    return NO_ERROR;
}
    
/******************************************************/
int sstBitBangWriteWord(u32 addr, u32 data,
    void (*setClock)(int), 
    void (*setData)(int), 
    int (*getData)(void),  
    int (*setCS)(void))
{
    setClockFunc = setClock;
    setDataFunc = setData;
    getDataFunc = getData;    
    setCSFunc = setCS;
    if (sstCmdWriteByte(addr++, (u08) data >> 24)==NO_ERROR)
    if (sstCmdWriteByte(addr++, (u08) data >> 16)==NO_ERROR)
    if (sstCmdWriteByte(addr++, (u08) data >> 8)==NO_ERROR)
    if (sstCmdWriteByte(addr  , (u08) data)==NO_ERROR)
        return NO_ERROR;
    return HAS_ERROR;
}


/****************************************************************************************/
/****************************************************************************************/
/*   ATMEL FLASH bit bang code                                                            */
/****************************************************************************************/
/****************************************************************************************/
#define FLASH_AML_CMD_CONTINUOUS_ARRAY_READ    0x68
#define FLASH_AML_CMD_BURST_ARRAY_READ         0x20

/******************************************************/
/*    Private Routine                                 */
/******************************************************/

/******************************************************/
/*    ATMEL Public Routine                            */
/******************************************************/
int atmlBitBangReadWord(u32 addr, u32* data,
    void (*setClock)(int), 
    void (*setData)(int), 
    int (*getData)(void),  
    void (*setCS)(void))
{
    int WordCnt=0x100000;
    
    setClockFunc = setClock;
    setDataFunc = setData;
    getDataFunc = getData;    
    setCSFunc = setCS;
    (setCSFunc)(1);                /* enable chip select */
    while (WordCnt--)
        *data++ = flshBangRead4();
    (setCSFunc)(0);
    return NO_ERROR;
}
    
/******************************************************/
int atmlBitBangWriteWord(u32 addr, u32 data, u32 first_last,
    void (*setClock)(int), 
    void (*setData)(int), 
    int (*getData)(void),  
    int (*setCS)(void))
{
    setClockFunc = setClock;
    setDataFunc = setData;
    getDataFunc = getData;    
    setCSFunc = setCS;
    if (first_last & FLSH_CMD_FIRST)
    {
        (setCSFunc)(1);                /* enable chip select */
    }
    
    if (first_last & FLSH_CMD_LAST)
    {
        (setCSFunc)(0);
    }
    return HAS_ERROR;
}
